package org.andorp.testkeys.annotation;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;

import org.andorp.testkeys.KeywordRepr;
import org.andorp.testkeys.KeywordCreationException;
import org.andorp.testkeys.KeywordFactory;
import org.andorp.testkeys.KeywordName;
import org.andorp.testkeys.KeywordRollback;
import org.andorp.testkeys.KeywordStep;

public final class AnnotedKeywordFactory implements KeywordFactory<Object>{

    public KeywordRepr createKeyword(Object value) throws KeywordCreationException {
        final Class<?> _class = value.getClass();
        
        final String nameStr = name(_class);
        try {
            final Method step = step(_class);
            final Method rollback = rollback(_class);
            
            if (nameStr == null) {
                throw new KeywordCreationException("No Keyword name was found");
            } 
            if (step == null) {
                throw new KeywordCreationException("No Keyword step was found");
            }
            if (rollback == null) {
                throw new KeywordCreationException("No Keyword rollback was found");
            }
            
            final KeywordName n     = new KeywordName(name(_class));
            final KeywordRollback r = new AKeywordRollback(value, rollback);
            final KeywordStep s     = new AKeywordStep(value, step);
            
            return new KeywordRepr(n,s,r);
        } catch (AKeywordFactoryException e) {
            throw new KeywordCreationException(e);
        }
    }
    
    private String name(Class<?> _class) {
        Name n = _class.getAnnotation(Name.class);
        if(n != null) {
            return n.name();
        } else {
            return null;
        }
    }
    
    private Method step(Class<?> _class) throws AKeywordFactoryException {
        Method stepMethod = null;
        final Method[] methods = _class.getMethods();
        for(Method m : methods) {
            final Annotation[] annotations = m.getDeclaredAnnotations();
            for(Annotation a : annotations) {
                if(a instanceof Step) {
                    if(stepMethod == null) {
                        if (m.getGenericReturnType().equals(Boolean.TYPE)) {
                            stepMethod = m;
                        }
                    } else {
                        throw new AKeywordFactoryException("More than one @Step annotation found");
                    } 
                }
            }
        }
        if(stepMethod == null) {
            throw new AKeywordFactoryException("No @Step annotation was found");
        }
        return stepMethod;
    }
    
    private Method rollback(Class<?> _class) throws AKeywordFactoryException {
        Method rollbackMethod = null;
        final Method[] methods = _class.getMethods();
        for(Method m : methods) {
            final Annotation[] annotations = m.getDeclaredAnnotations();
            for(Annotation a : annotations) {
                if(a instanceof Rollback) {
                    if(rollbackMethod == null) {
                       if(m.getGenericReturnType().equals(Void.TYPE)) {
                          rollbackMethod = m;;
                       } 
                    } else {
                       throw new AKeywordFactoryException("More than one @Rollback annotation was found");
                    }
                }
            }
        }
        if(rollbackMethod == null) {
            throw new AKeywordFactoryException("No @Rollback annotation was found");
        } 
        return rollbackMethod;
    }

}
