package org.andorp.testkeys.container;

import java.util.HashMap;

import org.andorp.testkeys.Keyword;
import org.andorp.testkeys.KeywordName;
import org.andorp.testkeys.KeywordRepr;
import org.andorp.testkeys.interfaced.KeywordObjectFactory;
import org.andorp.testkeys.story.ContainerException;
import org.andorp.testkeys.story.KeywordContainer;

public final class Standard implements KeywordContainer {

	private final HashMap<KeywordName, KeywordRepr> keywords;
	private final KeywordObjectFactory<Keyword> keywordFactory;
	
	public Standard() {
		keywords = new HashMap<KeywordName, KeywordRepr>();
		keywordFactory = new KeywordObjectFactory<Keyword>();
	}
	
	public KeywordRepr getKeyword(KeywordName name) throws ContainerException {
		if(!keywords.containsKey(name)) {
			throw new ContainerException("Keyword not found: " + name.toString());
		}
		return keywords.get(name);
	}
	
	public boolean contains(KeywordName name) {
		return keywords.containsKey(name);
	}
	
	public void register(Keyword k) throws ContainerException {
		try {
			final KeywordName n = k.name();
			if(keywords.containsKey(n)) {
				throw new ContainerException("The keyword is already registered in the container.");
			}
			final KeywordRepr r = keywordFactory.createKeyword(k);
			keywords.put(n, r);
		} catch (Exception e) {
			throw new ContainerException(e);
		}
	}
	
	public void unregister(KeywordName name) throws ContainerException {
		if(!keywords.containsKey(name)) {
			throw new ContainerException("Keyword not found: " + name.toString());
		}
		
	}
}
