package org.andorp.testkeys.story;

import java.util.ArrayList;
import java.util.List;

import org.andorp.testkeys.Keyword;
import org.andorp.testkeys.KeywordRepr;
import org.andorp.testkeys.KeywordName;
import org.andorp.testkeys.interfaced.KeywordObjectFactory;
import org.andorp.testkeys.story.Story;
import org.andorp.testkeys.test.CompositeKeyword;

public final class TestFactory {

    private final KeywordContainer keywordContainer;
    private final KeywordObjectFactory<Keyword> keywordFactory;
    
    public TestFactory(KeywordContainer _keywordContainer) {
        assert _keywordContainer != null;
        keywordContainer = _keywordContainer;
        keywordFactory = new KeywordObjectFactory<Keyword>(); 
    }
    
    public KeywordRepr createTestCase(Story story) throws ContainerException {
        try {
            if(story == null) {
                throw new ContainerException("Story was null");
            }
            if(story.isEmpty()) {
                throw new ContainerException("Story was empty");
            }
            
            final List<KeywordRepr> keywordSteps = new ArrayList<KeywordRepr>();
            for(KeywordName name : story.steps()) {
                keywordSteps.add(keywordContainer.getKeyword(name));
            }
            
            return keywordFactory.createKeyword(
                new CompositeKeyword(story.name(), keywordSteps)
            );
        } catch (Exception e) {
            throw new ContainerException(e);
        }
    }

}
