package org.andorp.testkeys.story.xml;

import java.io.InputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.andorp.testkeys.KeywordName;
import org.andorp.testkeys.story.Parser;
import org.andorp.testkeys.story.ParserException;
import org.andorp.testkeys.story.Story;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public final class XMLParser implements Parser<InputStream>{
    
    public Story parse(InputStream iStream) throws ParserException {
        
        try {
            
            // Story object to hold the steps
            final Story story = new Story();
            
            // XML parser initialization
            final DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
            final DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
            final Document doc = dBuilder.parse(iStream);
            
            // Parsing
            final String rootElementName = doc.getDocumentElement().getNodeName();
            
            if (rootElementName != "story") {
                throw new XmlParserException("Root element is not a story");
            }
            
            // Get the story name
            final String storyName = doc.getDocumentElement().getAttribute("name");
            if (storyName == null || storyName.isEmpty()) {
                throw new XmlParserException("Story name was not defined or empty string");
            }
            
            story.setName(new KeywordName(storyName));
            
            // Get the steps
            final NodeList nList = doc.getElementsByTagName("step");
            final int n = nList.getLength();
            
            for(int nodeIdx = 0; nodeIdx < n; ++nodeIdx) {
                final Node nNode = nList.item(nodeIdx);
                
                if (nNode.getNodeType() == Node.ELEMENT_NODE) {
                    final Element eElement = (Element)nNode;
                    final String content = eElement.getTextContent();
                    
                    // Add an appropiate step from the xml document
                    if (content != null && !content.isEmpty()) {
                        story.addStep(new KeywordName(content));
                    }
                }
                
            }
            
            if (story.isEmpty()) {
                throw new ParserException("No steps were found in the XML document");
            }
            
            return story;
            
        } catch (ParserException e) {
            throw e;
        } catch (Exception e) {
            throw new ParserException(e); 
        }
    }

}
