package org.andorp.testkeys.test;

import java.util.List;

import org.andorp.testkeys.Context;
import org.andorp.testkeys.KeywordRepr;
import org.andorp.testkeys.KeywordException;
import org.andorp.testkeys.KeywordName;
import org.andorp.testkeys.RollbackException;
import org.andorp.testkeys.Rollbacks;
import org.andorp.testkeys.Keyword;

public final class CompositeKeyword implements Keyword {
    
    private final List<KeywordRepr> steps;
    private final KeywordName name;
    final Rollbacks rollbacks = new Rollbacks();
    
    public CompositeKeyword(KeywordName _name, List<KeywordRepr> _steps) {
        steps = _steps;
        name = _name;
    }
    
    private boolean runTest(final Context c) throws KeywordException {
        final Context newFrame = c.newFrame();
        for(KeywordRepr k : steps) {
            // Try to run the test step
            rollbacks.add(k);
            final boolean success = k.step(newFrame);
            if (!success) {
                return false;
            }
        }
        return true;
    }
    
    private void runRollbacks(final Rollbacks rollbacks) throws RollbackException {
        rollbacks.runRollbacks();
    }

    @Override
    public KeywordName name() {
        return name;
    }

    @Override
    public boolean step(final Context c) throws KeywordException {
        return runTest(c);
    }

    @Override
    public void rollback() throws RollbackException {
        runRollbacks(rollbacks);
    }
    
}
