package org.andorp.testkeys.stage1;

import org.andorp.testkeys.KeywordCreationException;
import org.andorp.testkeys.annotation.AnnotedKeywordFactory;
import org.andorp.testkeys.annotation.Rollback;
import org.andorp.testkeys.annotation.Step;
import org.andorp.testkeys.annotation.Name;
import org.testng.Assert;
import org.testng.annotations.Test;

public final class TestAnnotationFactory {

    @Test(expectedExceptions = KeywordCreationException.class)
    public void noAnnotationsAtAll() throws KeywordCreationException {
       NoAnnotations objectUnderTest = new NoAnnotations();
       
       AnnotedKeywordFactory akf = new AnnotedKeywordFactory();
       akf.createKeyword(objectUnderTest);
    }
    
    @Test(expectedExceptions = KeywordCreationException.class)
    public void onlyStepAnnotation() throws KeywordCreationException {
       OnlyStepAnnotation objectUnderTest = new OnlyStepAnnotation();
       
       AnnotedKeywordFactory akf = new AnnotedKeywordFactory();
       akf.createKeyword(objectUnderTest);
    }
    
    @Test(expectedExceptions = KeywordCreationException.class)
    public void onlyRollbackAnnotation() throws KeywordCreationException {
       OnlyRollbackAnnotation objectUnderTest = new OnlyRollbackAnnotation();
       
       AnnotedKeywordFactory akf = new AnnotedKeywordFactory();
       akf.createKeyword(objectUnderTest);
    }
    
    @Test(expectedExceptions = KeywordCreationException.class)
    public void stepAndRollbackAnnotation() throws KeywordCreationException {
       StepAndRollbackAnnotation objectUnderTest = new StepAndRollbackAnnotation();
       
       AnnotedKeywordFactory akf = new AnnotedKeywordFactory();
       akf.createKeyword(objectUnderTest);
    }
    
    @Test(expectedExceptions = KeywordCreationException.class)
    public void multipleStepAnnotation() throws KeywordCreationException {
       MultipleStepsAnnotation objectUnderTest = new MultipleStepsAnnotation();
       
       AnnotedKeywordFactory akf = new AnnotedKeywordFactory();
       akf.createKeyword(objectUnderTest);
    }
    
    @Test(expectedExceptions = KeywordCreationException.class)
    public void multipleRollbackAnnotation() throws KeywordCreationException {
       MultipleRollbackAnnotation objectUnderTest = new MultipleRollbackAnnotation();
       
       AnnotedKeywordFactory akf = new AnnotedKeywordFactory();
       akf.createKeyword(objectUnderTest);
    }
    
    @Test()
    public void nameStepAndRollbackAnnotation() throws KeywordCreationException {
        NameStepAndRollbackAnnotation objectUnderTest = new NameStepAndRollbackAnnotation();
        
        AnnotedKeywordFactory akf = new AnnotedKeywordFactory();
        Object result = akf.createKeyword(objectUnderTest);
        Assert.assertNotNull(result, "Result object was null");
     }
    
    /* Test annotation classes */
    
    public static class NoAnnotations {
        
    }
    
    public static class OnlyStepAnnotation {
        
        @Step()
        public boolean step() {
            return false;
        }
        
    }
    
    public static class OnlyRollbackAnnotation {
        
        @Rollback()
        public void rollback() {
            return;
        }
    }
    
    public static class StepAndRollbackAnnotation {
        
        @Step()
        public boolean step() {
            return false;
        }
        
        @Rollback()
        public void rollback() {
            return;
        }
    }
    
    @Name(name="keywordname")
    public static class NameStepAndRollbackAnnotation {
        
        @Step()
        public boolean step() {
            return false;
        }
        
        @Rollback()
        public void rollback() {
            return;
        }
    }
    
    @Name(name="keywordname")
    public static class MultipleStepsAnnotation {
        
        @Step()
        public boolean step() {
            return false;
        }
        
        @Step()
        public boolean step2() {
            return false;
        }
        
        
        @Rollback()
        public void rollback() {
            return;
        }
    }
    
    @Name(name="keywordname")
    public static class MultipleRollbackAnnotation {
        
        @Step()
        public boolean step() {
            return false;
        }
        
        @Rollback()
        public void step2() {
            return;
        }
        
        
        @Rollback()
        public void rollback() {
            return;
        }
    }
    
    
}
